﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2015 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Linq;
using HEAL.Attic;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Encodings.SymbolicExpressionTreeEncoding;
using HeuristicLab.EvolutionTracking;

namespace HeuristicLab.Problems.DataAnalysis.Symbolic {
  [Item("SymbolicDataAnalysisExpressionAfterManipulatorOperator", "An operator that compares the mutation child with the parent and saves the modified fragment.")]
  [StorableType("8F3E48C0-1D06-429C-848C-CA4B5189440B")]
  public class SymbolicDataAnalysisExpressionAfterManipulatorOperator : AfterManipulatorOperator<ISymbolicExpressionTree> {
    public SymbolicDataAnalysisExpressionAfterManipulatorOperator() { }

    protected SymbolicDataAnalysisExpressionAfterManipulatorOperator(SymbolicDataAnalysisExpressionAfterManipulatorOperator original, Cloner cloner)
      : base(original, cloner) {
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new SymbolicDataAnalysisExpressionAfterManipulatorOperator(this, cloner);
    }

    [StorableConstructor]
    protected SymbolicDataAnalysisExpressionAfterManipulatorOperator(StorableConstructorFlag _) : base(_) { }

    public override IOperation Apply() {
      var child = ChildParameter.ActualValue;
      var childVertex = GenealogyGraph.GetByContent(child);
      var parent = childVertex.Parents.First().Data;
      var subtree = child.Difference(parent);
      Fragment<ISymbolicExpressionTreeNode> fragment = null;
      if (subtree != null) {
        int index = child.IterateNodesPrefix().TakeWhile(node => node != subtree).Count();
        fragment = new Fragment<ISymbolicExpressionTreeNode> { Root = subtree, Index1 = index, Index2 = index };
      }
      childVertex.InArcs.Last().Data = fragment;
      return base.Apply();
    }
  }
}
